#include <iostream>
#include "PositionExperiment.h"
#include "Position.h"
#include "IntegratorEuler.h"
#include "IntegratorTrapez.h"

using namespace std;

bool PositionExperiment::DoExperiment(char *argv[], const int argc,
									  const WiiLib::Vec3 *gravity,
									  const int samples,
									  const WiiLib::Vec3 *data,
									  double *output)
{
	if (argc < 4)
	{
		return false;
	}

	// Create convolution filter
	Filter *f;

	if (strcmp(argv[1], "gauss") == 0)
	{
		cout << "Gauss filter" << endl;
		cout << "  sigma = " << atof(argv[2]) << endl;
		f = Filter::CreateGaussFilter(atof(argv[2]));
	}
	else if (strcmp(argv[1], "pass") == 0)
	{
		cout << "No filter" << endl;
		f = Filter::CreatePassThroughFilter();
	}
	else
	{
		return false;
	}

	// Create position estimator
	Position pos(f, 0);
	IntegratorEuler integEuler;
	IntegratorTrapez integTrapez;

	if (strcmp(argv[0], "euler") == 0)
	{
		cout << "Euler integrator" << endl;
		pos.SetIntegrator(&integEuler);
	}
	else if (strcmp(argv[0], "trapez") == 0)
	{
		cout << "Trapez integrator" << endl;
		pos.SetIntegrator(&integTrapez);
	}
	else
	{
		return false;
	}

	// Run test
	double delta = atof(argv[3]);
	Vec3 result;

	cout << "Running test" << endl;
	for (int i = 0; i < samples; i++)
	{
		pos.ProcessSample(data[i], *gravity, delta);
		//Udls kun z-aksen, da kun denne pvirkes af bevgelse i vores "vogn"
		pos.GetPosition(result);
		output[i] = result.mZ;
	}

	return true;
}