#include <iostream>
#include "TiltExperiment.h"
#include "Tilt.h"
#include "GravityAdaptive.h"
#include "GravityNaive.h"

using namespace WiiLib;
using namespace std;

bool TiltExperiment::DoExperiment(char *argv[], const int argc,
								  const WiiLib::Vec3 *gravity, const int samples,
								  const WiiLib::Vec3 *data, double *output)
{
	if ((argc < 3) || ((strcmp(argv[0], "adaptive") == 0) && (argc < 5)))
	{
		return false;
	}

	// Create convolution filter
	Filter *f;

	if (strcmp(argv[3], "gauss") == 0)
	{
		cout << "Gauss filter" << endl;
		cout << "  sigma = " << atof(argv[4]) << endl;
		f = Filter::CreateGaussFilter(atof(argv[4]));
	}
	else if (strcmp(argv[3], "pass") == 0)
	{
		cout << "No filter" << endl;
		f = Filter::CreatePassThroughFilter();
	}
	else
	{
		return false;
	}

	// Create tilt estimator
	Tilt tilt(f, 0);
	tilt.SetReference(*gravity);
	GravityAdaptive gmAdaptive(*gravity, atof(argv[2]), atof(argv[1]));
	GravityNaive gmNaive;

	if (strcmp(argv[0], "adaptive") == 0)
	{
		cout << "Adaptive method" << endl;
		cout << "  beta = " << atof(argv[2]) << endl;
		cout << "  k = " << atof(argv[1]) << endl;
		tilt.SetGravityModel(&gmAdaptive);
	}
	else if (strcmp(argv[0], "naive") == 0)
	{
		cout << "Naive method" << endl;
		tilt.SetGravityModel(&gmNaive);
	}
	else
	{
		return false;
	}

	// Run test
	Vec3 dummy;
	double angle = 0.0;
	
	cout << "Running test" << endl;
	for (int i = 0; i < samples; i++)
	{
		tilt.ProcessSample(data[i]);
		tilt.GetAxisAngle(dummy, output[i]);
	}

	return true;
}