#include <fstream>
#include <iostream>
#include <cmath>
#include <time.h>
#include <String>

#include "Vec3.h"
#include "Experiment.h"
#include "KExperiment.h"
#include "BExperiment.h"
#include "TiltExperiment.h"
#include "PositionExperiment.h"

#pragma comment(lib, "WiiLib.lib")

using namespace std;
using namespace WiiLib;

bool ReadSamples(const char *file, Vec3 &gravity, int &samples, Vec3 *&container)
{
	ifstream fin;

	// Create data file
	fin.open(file);
	if (!fin.is_open())
	{
		return false;
	}

	fin >> gravity.mX;
	fin >> gravity.mY;
	fin >> gravity.mZ;

	fin >> samples;
	double dummy;
	fin >> dummy;
	fin >> dummy;

	//Try catch?
	container = new Vec3[samples];
	for (int i = 0; i < samples; ++i)
	{
		container[i] = Vec3();
		fin >> container[i].mX;
		fin >> container[i].mY;
		fin >> container[i].mZ;
	}

	fin.close();
	return true;
}

bool WriteSamples(const char *file, const Vec3 gravity, const int samples,
				  const double *data)
{
	ofstream fout;

	fout.open(file, ofstream::trunc);
	if (!fout.is_open())
	{
		return false;
	}
	else
	{
		fout.precision(16);
	}

	for (int i = 0; i < samples; i++)
	{
		fout << scientific << data[i] << endl;
	}

	fout.close();

	return true;
}

int main(int argc, char *argv[])
{
	Vec3 gravity;
	Vec3 *data;
	int samples = 0;
	int argCnt = argc - 4;
	bool success = false;
	double *output;
	string outfile;

	if (argc < 3)
	{
		cout << "Syntax: " << argv[0]
			<< " <input filename> <output filename> <test-type> [options]"
			<< endl;
		return 0;
	}

	if (!ReadSamples(argv[1], gravity, samples, data))
	{
		return 0;
	}

	//Work start

	output = new double[samples];
	memset(output, 0, sizeof(double)*samples);
	if (strcmp(argv[3], "k-test") == 0)
	{
		KExperiment kex = KExperiment();
		success = kex.DoExperiment(&argv[4], argCnt, &gravity, samples, data,
									output);
	}
	else if (strcmp(argv[3], "b-test") == 0)
	{
		BExperiment bex = BExperiment();
		success = bex.DoExperiment(&argv[4], argCnt, &gravity, samples, data,
									output);
	}
	else if (strcmp(argv[3], "tilt") == 0)
	{
		TiltExperiment TE = TiltExperiment();
		success = TE.DoExperiment(&argv[4], argCnt, &gravity, samples, data,
									output);
	}
	else if (strcmp(argv[3], "position") == 0)
	{
		PositionExperiment PE = PositionExperiment();
		success = PE.DoExperiment(&argv[4], argCnt, &gravity, samples, data,
									output);
	}
	else
	{
		delete [] output;
		return 0;
	}

	outfile = argv[2];
	for (int i = 3; i < argc; ++i)
	{
		outfile += "_";
		outfile += argv[i];
	}
	outfile += ".mat";

	if (success && WriteSamples(outfile.c_str(), gravity, samples, output))
	{
		cout << "Successfully done!" << endl;
	}
	else
	{
		cout << "What arrrrrr' yo' doin'?!" << endl;
	}

	//Work end

	delete [] data;
	delete [] output;

	return 0;
}
